// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"strings"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
)

func ProjectInfo(ctx context.Context, req *v1.ProjectInfoRequest) (reply *v1.ProjectInfoReply, err error) {
	if req.Id < 1 {
		return nil, errors.ParamsError
	}
	where := map[string]interface{}{
		"id": req.Id,
	}
	p := model.TProject{}
	p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.Id == 0 {
		return nil, errors.NoRecordError
	}
	reply = &v1.ProjectInfoReply{}
	reply.Info = &v1.ProjectItem{}
	reply.Info.Id = p.Id
	reply.Info.Location = p.Location
	reply.Info.Name = p.Name
	reply.Info.Phone = p.Phone
	reply.Info.SafetyRecordNo = p.SafetyRecordNo
	reply.Info.ProjectLeader = p.ProjectLeader
	reply.Info.ProjectNo = p.ProjectNo
	reply.Info.Lon = p.Lon
	reply.Info.Lat = p.Lat
	reply.Info.Price = p.Price
	reply.Info.BuildingStorey = p.BuildingStorey
	reply.Info.Construction = p.Construction
	reply.Info.CoveredArea = p.CoveredArea
	reply.Info.Desc = p.Desc
	reply.Info.Design = p.Design
	reply.Info.Development = p.Development
	reply.Info.Exploration = p.Exploration
	reply.Info.Oversee = p.Oversee
	reply.Info.ProjectFunction = p.ProjectFunction
	reply.Info.ProjectPlan = p.ProjectPlan
	reply.Info.ProjectType = p.ProjectType
	reply.Info.TotalLevel = int32(p.TotalLevel)
	reply.Info.StartDay = p.StartDay.Format("2006-01-02")
	reply.Info.EndDay = p.EndDay.Format("2006-01-02")
	reply.Info.ProjectEffectPic = p.ProjectEffectPic
	reply.Info.CreateTime = p.CreatedAt.Format("2006-01-02 15:04:05")
	if p.IsFinish > 0 {
		reply.Info.IsFinish = true
	}
	if !strings.Contains(p.ProjectPlan, "[") {
		reply.Info.ProjectPlan = "[]"
	}
	if !strings.Contains(p.ProjectEffectPic, "[") {
		reply.Info.ProjectEffectPic = "[]"
	}

	return reply, nil
}
